@@ -1,10 +1,10 @@
 /*-------------------------------------------------------------------------
 *
-* Copyright (c) 2004-2005, PostgreSQL Global Development Group
+* Copyright (c) 2004-2008, PostgreSQL Global Development Group
 * Copyright (c) 2004, Open Cloud Limited.
 *
 * IDENTIFICATION
-*        $PostgreSQL: pgjdbc/org/postgresql/core/Utils.java,v 1.5 2006/12/01 08:53:45 jurka Exp $
+*        $PostgreSQL: pgjdbc/org/postgresql/core/Utils.java,v 1.8 2008/09/19 23:44:32 jurka Exp $
 *
 *-------------------------------------------------------------------------
 */
@@ -13,6 +13,10 @@ package org.postgresql.core;
 
 import java.sql.SQLException;
 
+import java.nio.ByteBuffer;
+import java.nio.CharBuffer;
+import java.nio.charset.Charset;
+
 import org.postgresql.util.GT;
 import org.postgresql.util.PSQLException;
 import org.postgresql.util.PSQLState;
@@ -38,6 +42,13 @@ public class Utils {
         return sb.toString();
     }
 
+    /**
+     * Keep a local copy of the UTF-8 Charset so we can avoid
+     * synchronization overhead from looking up the Charset by
+     * name as String.getBytes(String) requires.
+     */
+    private final static Charset utf8Charset = Charset.forName("UTF-8");
+
     /**
      * Encode a string as UTF-8.
      *
@@ -45,23 +56,20 @@ public class Utils {
      * @return the UTF-8 representation of <code>str</code>
      */
     public static byte[] encodeUTF8(String str) {
-        // It turns out that under 1.4.2, at least, calling getBytes() is
-        // faster than rolling our own converter (it uses direct buffers and, I suspect,
-        // a native helper -- and the cost of the encoding lookup is mitigated by a
-        // ThreadLocal that caches the last lookup). So we just do things the simple way here.
-        try
-        {
-            return str.getBytes("UTF-8");
-        }
-        catch (java.io.UnsupportedEncodingException e)
-        {
-            // Javadoc says that UTF-8 *must* be supported by all JVMs, so we don't try to be clever here.
-            throw new RuntimeException("Unexpected exception: UTF-8 charset not supported: " + e);
-        }
+        // Previously we just used str.getBytes("UTF-8"), but when
+        // the JVM is using more than one encoding the lookup cost
+        // makes that a loser to the below (even in the single thread case).
+        // When multiple threads are doing Charset lookups, they all get
+        // blocked and must wait, severely dropping throughput.
+        //
+        ByteBuffer buf = utf8Charset.encode(CharBuffer.wrap(str));
+        byte b[] = new byte[buf.limit()];
+        buf.get(b, 0, buf.limit());
+        return b;
     }
-    
+
     /**
-     * Escape the given string <tt>value</tt> and append it to the string buffer
+     * Escape the given literal <tt>value</tt> and append it to the string buffer
      * <tt>sbuf</tt>. If <tt>sbuf</tt> is <tt>null</tt>, a new StringBuffer will be
      * returned. The argument <tt>standardConformingStrings</tt> defines whether the
      * backend expects standard-conforming string literals or allows backslash
@@ -73,7 +81,7 @@ public class Utils {
      * @return the sbuf argument; or a new string buffer for sbuf == null
      * @throws SQLException if the string contains a <tt>\0</tt> character
      */
-    public static StringBuffer appendEscapedString(StringBuffer sbuf, String value,
+    public static StringBuffer appendEscapedLiteral(StringBuffer sbuf, String value,
                                                    boolean standardConformingStrings)
                                                    throws SQLException {
         if (sbuf == null)
@@ -112,4 +120,38 @@ public class Utils {
         
         return sbuf;
     }
+
+    /**
+     * Escape the given identifier <tt>value</tt> and append it to the string
+     * buffer * <tt>sbuf</tt>. If <tt>sbuf</tt> is <tt>null</tt>, a new
+     * StringBuffer will be returned.  This method is different from
+     * appendEscapedLiteral in that it includes the quoting required for the
+     * identifier while appendEscapedLiteral does not.
+     * 
+     * @param sbuf the string buffer to append to; or <tt>null</tt>
+     * @param value the string value
+     * @return the sbuf argument; or a new string buffer for sbuf == null
+     * @throws SQLException if the string contains a <tt>\0</tt> character
+     */
+    public static StringBuffer appendEscapedIdentifier(StringBuffer sbuf, String value)
+                                                   throws SQLException {
+        if (sbuf == null)
+            sbuf = new StringBuffer(2 + value.length() * 11 / 10); // Add 10% for escaping.
+
+        sbuf.append('"');
+
+        for (int i = 0; i < value.length(); ++i)
+        {
+            char ch = value.charAt(i);
+            if (ch == '\0')
+                throw new PSQLException(GT.tr("Zero bytes may not occur in identifiers."), PSQLState.INVALID_PARAMETER_VALUE);
+            if (ch == '"')
+                sbuf.append(ch);
+            sbuf.append(ch);
+        }
+
+        sbuf.append('"');
+
+        return sbuf;
+    }
 }
