@@ -1,10 +1,10 @@
 /*-------------------------------------------------------------------------
 *
-* Copyright (c) 2003-2005, PostgreSQL Global Development Group
+* Copyright (c) 2003-2008, PostgreSQL Global Development Group
 * Copyright (c) 2004, Open Cloud Limited.
 *
 * IDENTIFICATION
-*   $PostgreSQL: pgjdbc/org/postgresql/core/v2/ConnectionFactoryImpl.java,v 1.10 2006/12/01 08:53:45 jurka Exp $
+*   $PostgreSQL: pgjdbc/org/postgresql/core/v2/ConnectionFactoryImpl.java,v 1.18 2009/06/02 00:22:58 jurka Exp $
 *
 *-------------------------------------------------------------------------
 */
@@ -46,16 +46,12 @@ public class ConnectionFactoryImpl extends ConnectionFactory {
         boolean requireSSL = (info.getProperty("ssl") != null);
         boolean trySSL = requireSSL; // XXX temporary until we revisit the ssl property values
 
+        //  - the TCP keep alive setting
+        boolean requireTCPKeepAlive = (Boolean.valueOf(info.getProperty("tcpKeepAlive")).booleanValue());
+
         if (logger.logDebug())
             logger.debug("Trying to establish a protocol version 2 connection to " + host + ":" + port);
 
-        if (!Driver.sslEnabled())
-        {
-            if (requireSSL)
-                throw new PSQLException(GT.tr("The driver does not support SSL."), PSQLState.CONNECTION_FAILURE);
-            trySSL = false;
-        }
-
         //
         // Establish a connection.
         //
@@ -68,6 +64,22 @@ public class ConnectionFactoryImpl extends ConnectionFactory {
             // Construct and send an ssl startup packet if requested.
             if (trySSL)
                 newStream = enableSSL(newStream, requireSSL, info, logger);
+            
+            
+            // Set the socket timeout if the "socketTimeout" property has been set.
+            String socketTimeoutProperty = info.getProperty("socketTimeout", "0");
+            try {
+                int socketTimeout = Integer.parseInt(socketTimeoutProperty);
+                if (socketTimeout > 0) {
+                    newStream.getSocket().setSoTimeout(socketTimeout*1000);
+                }
+            } catch (NumberFormatException nfe) {
+                logger.info("Couldn't parse socketTimeout value:" + socketTimeoutProperty);
+            }
+
+
+            // Enable TCP keep-alive probe if required.
+            newStream.getSocket().setKeepAlive(requireTCPKeepAlive);
 
             // Construct and send a startup packet.
             sendStartupPacket(newStream, user, database, logger);
@@ -165,7 +177,7 @@ public class ConnectionFactoryImpl extends ConnectionFactory {
                 logger.debug(" <=BE SSLOk");
 
             // Server supports ssl
-            Driver.makeSSL(pgStream, info, logger);
+            org.postgresql.ssl.MakeSSL.convert(pgStream, info, logger);
             return pgStream;
 
         default:
@@ -189,8 +201,8 @@ public class ConnectionFactoryImpl extends ConnectionFactory {
         pgStream.SendInteger4(4 + 4 + 64 + 32 + 64 + 64 + 64);
         pgStream.SendInteger2(2); // protocol major
         pgStream.SendInteger2(0); // protocol minor
-        pgStream.Send(database.getBytes("US-ASCII"), 64);
-        pgStream.Send(user.getBytes("US-ASCII"), 32);
+        pgStream.Send(database.getBytes("UTF-8"), 64);
+        pgStream.Send(user.getBytes("UTF-8"), 32);
         pgStream.Send(new byte[64]);  // options
         pgStream.Send(new byte[64]);  // unused
         pgStream.Send(new byte[64]);  // tty
@@ -223,26 +235,25 @@ public class ConnectionFactoryImpl extends ConnectionFactory {
             case 'R':
                 // Authentication request.
                 // Get the type of request
-                int areq = pgStream.ReceiveIntegerR(4);
+                int areq = pgStream.ReceiveInteger4();
 
                 // Process the request.
                 switch (areq)
                 {
                 case AUTH_REQ_CRYPT:
                     {
-                        String salt = pgStream.ReceiveString(2);
+                        byte salt[] = pgStream.Receive(2);
 
                         if (logger.logDebug())
-                            logger.debug(" <=BE AuthenticationReqCrypt(salt='" + salt + "')");
+                            logger.debug(" <=BE AuthenticationReqCrypt(salt='" + new String(salt, "US-ASCII") + "')");
 
                         if (password == null)
                             throw new PSQLException(GT.tr("The server requested password-based authentication, but no password was provided."), PSQLState.CONNECTION_REJECTED);
 
-                        String result = UnixCrypt.crypt(salt, password);
-                        byte[] encodedResult = result.getBytes("US-ASCII");
+                        byte[] encodedResult = UnixCrypt.crypt(salt, password.getBytes("UTF-8"));
 
                         if (logger.logDebug())
-                            logger.debug(" FE=> Password(crypt='" + result + "')");
+                            logger.debug(" FE=> Password(crypt='" + new String(encodedResult, "US-ASCII") + "')");
 
                         pgStream.SendInteger4(4 + encodedResult.length + 1);
                         pgStream.Send(encodedResult);
@@ -261,7 +272,7 @@ public class ConnectionFactoryImpl extends ConnectionFactory {
                         if (password == null)
                             throw new PSQLException(GT.tr("The server requested password-based authentication, but no password was provided."), PSQLState.CONNECTION_REJECTED);
 
-                        byte[] digest = MD5Digest.encode(user, password, md5Salt);
+                        byte[] digest = MD5Digest.encode(user.getBytes("UTF-8"), password.getBytes("UTF-8"), md5Salt);
                         if (logger.logDebug())
                             logger.debug(" FE=> Password(md5digest=" + new String(digest, "US-ASCII") + ")");
 
@@ -284,7 +295,7 @@ public class ConnectionFactoryImpl extends ConnectionFactory {
                         if (logger.logDebug())
                             logger.debug(" FE=> Password(password=<not shown>)");
 
-                        byte[] encodedPassword = password.getBytes("US-ASCII");
+                        byte[] encodedPassword = password.getBytes("UTF-8");
                         pgStream.SendInteger4(4 + encodedPassword.length + 1);
                         pgStream.Send(encodedPassword);
                         pgStream.SendChar(0);
@@ -326,8 +337,8 @@ public class ConnectionFactoryImpl extends ConnectionFactory {
                 return ;
 
             case 'K':  // BackendKeyData
-                int pid = pgStream.ReceiveIntegerR(4);
-                int ckey = pgStream.ReceiveIntegerR(4);
+                int pid = pgStream.ReceiveInteger4();
+                int ckey = pgStream.ReceiveInteger4();
                 if (logger.logDebug())
                     logger.debug(" <=BE BackendKeyData(pid=" + pid + ",ckey=" + ckey + ")");
                 protoConnection.setBackendKeyData(pid, ckey);
@@ -352,7 +363,7 @@ public class ConnectionFactoryImpl extends ConnectionFactory {
         }
     }
 
-    private class SimpleResultHandler implements ResultHandler {
+    private static class SimpleResultHandler implements ResultHandler {
         private SQLException error;
         private Vector tuples;
         private final ProtocolConnectionImpl protoConnection;
@@ -439,7 +450,13 @@ public class ConnectionFactoryImpl extends ConnectionFactory {
             if (logger.logDebug())
                 logger.debug("Switching to UNICODE client_encoding");
 
-            runSetupQuery(protoConnection, "begin; set autocommit = on; set client_encoding = 'UNICODE'; commit", false);
+            String sql = "begin; set autocommit = on; set client_encoding = 'UNICODE'; ";
+            if (dbVersion.compareTo("7.4") >= 0) {
+                sql += "SET extra_float_digits=2; ";
+            }
+            sql += "commit";
+
+            runSetupQuery(protoConnection, sql, false);
             protoConnection.setEncoding(Encoding.getDatabaseEncoding("UNICODE"));
         }
         else
