@@ -1,10 +1,10 @@
 /*-------------------------------------------------------------------------
 *
-* Copyright (c) 2003-2005, PostgreSQL Global Development Group
+* Copyright (c) 2003-2008, PostgreSQL Global Development Group
 * Copyright (c) 2004, Open Cloud Limited.
 *
 * IDENTIFICATION
-*   $PostgreSQL: pgjdbc/org/postgresql/core/v2/QueryExecutorImpl.java,v 1.18 2006/12/01 08:53:45 jurka Exp $
+*   $PostgreSQL: pgjdbc/org/postgresql/core/v2/QueryExecutorImpl.java,v 1.22 2009/07/01 05:00:40 jurka Exp $
 *
 *-------------------------------------------------------------------------
 */
@@ -19,6 +19,7 @@ import org.postgresql.core.*;
 import org.postgresql.util.PSQLException;
 import org.postgresql.util.PSQLState;
 import org.postgresql.util.GT;
+import org.postgresql.copy.CopyOperation;
 
 /**
  * QueryExecutor implementation for the V2 protocol.
@@ -111,7 +112,7 @@ public class QueryExecutorImpl implements QueryExecutor {
                 V2Query query = (V2Query)createSimpleQuery("");
                 SimpleParameterList params = (SimpleParameterList)query.createParameterList();
                 sendQuery(query, params, "BEGIN");
-                processResults(query, handler, 0);
+                processResults(query, handler, 0, 0);
             }
             catch (IOException ioe)
             {
@@ -211,7 +212,7 @@ public class QueryExecutorImpl implements QueryExecutor {
                         logger.debug(" <=BE FastpathResult");
 
                     // Result.
-                    int len = pgStream.ReceiveIntegerR(4);
+                    int len = pgStream.ReceiveInteger4();
                     result = pgStream.Receive(len);
                     c = pgStream.ReceiveChar();
                 }
@@ -361,7 +362,7 @@ public class QueryExecutorImpl implements QueryExecutor {
         try
         {
             sendQuery(query, parameters, queryPrefix);
-            processResults(query, handler, maxRows);
+            processResults(query, handler, maxRows, flags);
         }
         catch (IOException e)
         {
@@ -398,7 +399,8 @@ public class QueryExecutorImpl implements QueryExecutor {
         pgStream.flush();
     }
 
-    protected void processResults(Query originalQuery, ResultHandler handler, int maxRows) throws IOException {
+    protected void processResults(Query originalQuery, ResultHandler handler, int maxRows, int flags) throws IOException {
+        boolean bothRowsAndStatus = (flags & QueryExecutor.QUERY_BOTH_ROWS_AND_STATUS) != 0;
         Field[] fields = null;
         Vector tuples = null;
 
@@ -447,6 +449,9 @@ public class QueryExecutorImpl implements QueryExecutor {
                 {
                     handler.handleResultRows(originalQuery, fields, tuples, null);
                     fields = null;
+
+                    if (bothRowsAndStatus)
+                        interpretCommandStatus(status, handler);
                 }
                 else
                 {
@@ -522,7 +527,7 @@ public class QueryExecutorImpl implements QueryExecutor {
      */
     private Field[] receiveFields() throws IOException
     {
-        int size = pgStream.ReceiveIntegerR(2);
+        int size = pgStream.ReceiveInteger2();
         Field[] fields = new Field[size];
 
         if (logger.logDebug())
@@ -531,9 +536,9 @@ public class QueryExecutorImpl implements QueryExecutor {
         for (int i = 0; i < fields.length; i++)
         {
             String columnLabel = pgStream.ReceiveString();
-            int typeOid = pgStream.ReceiveIntegerR(4);
-            int typeLength = pgStream.ReceiveIntegerR(2);
-            int typeModifier = pgStream.ReceiveIntegerR(4);
+            int typeOid = pgStream.ReceiveInteger4();
+            int typeLength = pgStream.ReceiveInteger2();
+            int typeModifier = pgStream.ReceiveInteger4();
             fields[i] = new Field(columnLabel, columnLabel, typeOid, typeLength, typeModifier, 0, 0);
         }
 
@@ -541,7 +546,7 @@ public class QueryExecutorImpl implements QueryExecutor {
     }
 
     private void receiveAsyncNotify() throws IOException {
-        int pid = pgStream.ReceiveIntegerR(4);
+        int pid = pgStream.ReceiveInteger4();
         String msg = pgStream.ReceiveString();
 
         if (logger.logDebug())
@@ -601,5 +606,8 @@ public class QueryExecutorImpl implements QueryExecutor {
     private final ProtocolConnectionImpl protoConnection;
     private final PGStream pgStream;
     private final Logger logger;
-}
 
+    public CopyOperation startCopy(String sql) throws SQLException {
+        throw new PSQLException(GT.tr("Copy not implemented for protocol version 2"), PSQLState.NOT_IMPLEMENTED);
+    }
+}
