@@ -1,10 +1,10 @@
 /*-------------------------------------------------------------------------
 *
-* Copyright (c) 2003-2005, PostgreSQL Global Development Group
+* Copyright (c) 2003-2008, PostgreSQL Global Development Group
 * Copyright (c) 2004, Open Cloud Limited.
 *
 * IDENTIFICATION
-*   $PostgreSQL: pgjdbc/org/postgresql/core/v3/ConnectionFactoryImpl.java,v 1.12 2006/12/01 08:53:45 jurka Exp $
+*   $PostgreSQL: pgjdbc/org/postgresql/core/v3/ConnectionFactoryImpl.java,v 1.20 2009/06/02 00:22:58 jurka Exp $
 *
 *-------------------------------------------------------------------------
 */
@@ -39,6 +39,9 @@ public class ConnectionFactoryImpl extends ConnectionFactory {
     private static final int AUTH_REQ_CRYPT = 4;
     private static final int AUTH_REQ_MD5 = 5;
     private static final int AUTH_REQ_SCM = 6;
+    private static final int AUTH_REQ_GSS = 7;
+    private static final int AUTH_REQ_GSS_CONTINUE = 8;
+    private static final int AUTH_REQ_SSPI = 9;
 
     /** Marker exception; thrown when we want to fall back to using V2. */
     private static class UnsupportedProtocolException extends IOException {
@@ -50,6 +53,9 @@ public class ConnectionFactoryImpl extends ConnectionFactory {
         boolean requireSSL = (info.getProperty("ssl") != null);
         boolean trySSL = requireSSL; // XXX temporary until we revisit the ssl property values
 
+        //  - the TCP keep alive setting
+        boolean requireTCPKeepAlive = (Boolean.valueOf(info.getProperty("tcpKeepAlive")).booleanValue());
+
         // NOTE: To simplify this code, it is assumed that if we are
         // using the V3 protocol, then the database is at least 7.4.  That
         // eliminates the need to check database versions and maintain
@@ -60,13 +66,6 @@ public class ConnectionFactoryImpl extends ConnectionFactory {
         if (logger.logDebug())
             logger.debug("Trying to establish a protocol version 3 connection to " + host + ":" + port);
 
-        if (!Driver.sslEnabled())
-        {
-            if (requireSSL)
-                throw new PSQLException(GT.tr("The driver does not support SSL."), PSQLState.CONNECTION_FAILURE);
-            trySSL = false;
-        }
-
         //
         // Establish a connection.
         //
@@ -79,19 +78,34 @@ public class ConnectionFactoryImpl extends ConnectionFactory {
             // Construct and send an ssl startup packet if requested.
             if (trySSL)
                 newStream = enableSSL(newStream, requireSSL, info, logger);
+            
+            // Set the socket timeout if the "socketTimeout" property has been set.
+            String socketTimeoutProperty = info.getProperty("socketTimeout", "0");
+            try {
+                int socketTimeout = Integer.parseInt(socketTimeoutProperty);
+                if (socketTimeout > 0) {
+                    newStream.getSocket().setSoTimeout(socketTimeout*1000);
+                }
+            } catch (NumberFormatException nfe) {
+                logger.info("Couldn't parse socketTimeout value:" + socketTimeoutProperty);
+            }
+
+            // Enable TCP keep-alive probe if required.
+            newStream.getSocket().setKeepAlive(requireTCPKeepAlive);
 
             // Construct and send a startup packet.
             String[][] params = {
                                     { "user", user },
                                     { "database", database },
                                     { "client_encoding", "UNICODE" },
-                                    { "DateStyle", "ISO" }
+                                    { "DateStyle", "ISO" },
+                                    { "extra_float_digits", "2" }
                                 };
 
             sendStartupPacket(newStream, params, logger);
 
             // Do authentication (until AuthenticationOk).
-            doAuthentication(newStream, user, info.getProperty("password"), logger);
+            doAuthentication(newStream, host, user, info, logger);
 
             // Do final startup.
             ProtocolConnectionImpl protoConnection = new ProtocolConnectionImpl(newStream, user, database, info, logger);
@@ -192,7 +206,7 @@ public class ConnectionFactoryImpl extends ConnectionFactory {
                 logger.debug(" <=BE SSLOk");
 
             // Server supports ssl
-            Driver.makeSSL(pgStream, info, logger);
+            org.postgresql.ssl.MakeSSL.convert(pgStream, info, logger);
             return pgStream;
 
         default:
@@ -220,8 +234,8 @@ public class ConnectionFactoryImpl extends ConnectionFactory {
         byte[][] encodedParams = new byte[params.length * 2][];
         for (int i = 0; i < params.length; ++i)
         {
-            encodedParams[i*2] = params[i][0].getBytes("US-ASCII");
-            encodedParams[i*2 + 1] = params[i][1].getBytes("US-ASCII");
+            encodedParams[i*2] = params[i][0].getBytes("UTF-8");
+            encodedParams[i*2 + 1] = params[i][1].getBytes("UTF-8");
             length += encodedParams[i * 2].length + 1 + encodedParams[i * 2 + 1].length + 1;
         }
 
@@ -243,11 +257,13 @@ public class ConnectionFactoryImpl extends ConnectionFactory {
         pgStream.flush();
     }
 
-    private void doAuthentication(PGStream pgStream, String user, String password, Logger logger) throws IOException, SQLException
+    private void doAuthentication(PGStream pgStream, String host, String user, Properties info, Logger logger) throws IOException, SQLException
     {
         // Now get the response from the backend, either an error message
         // or an authentication request
 
+        String password = info.getProperty("password");
+
         while (true)
         {
             int beresp = pgStream.ReceiveChar();
@@ -261,7 +277,7 @@ public class ConnectionFactoryImpl extends ConnectionFactory {
                 // The most common one to be thrown here is:
                 // "User authentication failed"
                 //
-                int l_elen = pgStream.ReceiveIntegerR(4);
+                int l_elen = pgStream.ReceiveInteger4();
                 if (l_elen > 30000)
                 {
                     // if the error length is > than 30000 we assume this is really a v2 protocol
@@ -277,32 +293,28 @@ public class ConnectionFactoryImpl extends ConnectionFactory {
             case 'R':
                 // Authentication request.
                 // Get the message length
-                int l_msgLen = pgStream.ReceiveIntegerR(4);
+                int l_msgLen = pgStream.ReceiveInteger4();
 
                 // Get the type of request
-                int areq = pgStream.ReceiveIntegerR(4);
+                int areq = pgStream.ReceiveInteger4();
 
                 // Process the request.
                 switch (areq)
                 {
                 case AUTH_REQ_CRYPT:
                     {
-                        byte[] rst = new byte[2];
-                        rst[0] = (byte)pgStream.ReceiveChar();
-                        rst[1] = (byte)pgStream.ReceiveChar();
-                        String salt = new String(rst, 0, 2, "US-ASCII");
+                        byte[] salt = pgStream.Receive(2);
 
                         if (logger.logDebug())
-                            logger.debug(" <=BE AuthenticationReqCrypt(salt='" + salt + "')");
+                            logger.debug(" <=BE AuthenticationReqCrypt(salt='" + new String(salt, "US-ASCII") + "')");
 
                         if (password == null)
                             throw new PSQLException(GT.tr("The server requested password-based authentication, but no password was provided."), PSQLState.CONNECTION_REJECTED);
 
-                        String result = UnixCrypt.crypt(salt, password);
-                        byte[] encodedResult = result.getBytes("US-ASCII");
+                        byte[] encodedResult = UnixCrypt.crypt(salt, password.getBytes("UTF-8"));
 
                         if (logger.logDebug())
-                            logger.debug(" FE=> Password(crypt='" + result + "')");
+                            logger.debug(" FE=> Password(crypt='" + new String(encodedResult, "US-ASCII") + "')");
 
                         pgStream.SendChar('p');
                         pgStream.SendInteger4(4 + encodedResult.length + 1);
@@ -324,7 +336,7 @@ public class ConnectionFactoryImpl extends ConnectionFactory {
                         if (password == null)
                             throw new PSQLException(GT.tr("The server requested password-based authentication, but no password was provided."), PSQLState.CONNECTION_REJECTED);
 
-                        byte[] digest = MD5Digest.encode(user, password, md5Salt);
+                        byte[] digest = MD5Digest.encode(user.getBytes("UTF-8"), password.getBytes("UTF-8"), md5Salt);
 
                         if (logger.logDebug())
                         {
@@ -351,7 +363,7 @@ public class ConnectionFactoryImpl extends ConnectionFactory {
                         if (password == null)
                             throw new PSQLException(GT.tr("The server requested password-based authentication, but no password was provided."), PSQLState.CONNECTION_REJECTED);
 
-                        byte[] encodedPassword = password.getBytes("US-ASCII");
+                        byte[] encodedPassword = password.getBytes("UTF-8");
 
                         pgStream.SendChar('p');
                         pgStream.SendInteger4(4 + encodedPassword.length + 1);
@@ -362,6 +374,15 @@ public class ConnectionFactoryImpl extends ConnectionFactory {
                         break;
                     }
 
+                case AUTH_REQ_GSS:
+                    org.postgresql.gss.MakeGSS.authenticate(pgStream, host,
+                            user, password, 
+                            info.getProperty("jaasApplicationName"),
+                            info.getProperty("kerberosServerName"),
+                            logger);
+                    break;
+
+
                 case AUTH_REQ_OK:
                     if (logger.logDebug())
                         logger.debug(" <=BE AuthenticationOk");
@@ -391,7 +412,7 @@ public class ConnectionFactoryImpl extends ConnectionFactory {
             {
             case 'Z':
                 // Ready For Query; we're done.
-                if (pgStream.ReceiveIntegerR(4) != 5)
+                if (pgStream.ReceiveInteger4() != 5)
                     throw new IOException("unexpected length of ReadyForQuery packet");
 
                 char tStatus = (char)pgStream.ReceiveChar();
@@ -419,12 +440,12 @@ public class ConnectionFactoryImpl extends ConnectionFactory {
 
             case 'K':
                 // BackendKeyData
-                int l_msgLen = pgStream.ReceiveIntegerR(4);
+                int l_msgLen = pgStream.ReceiveInteger4();
                 if (l_msgLen != 12)
                     throw new PSQLException(GT.tr("Protocol error.  Session setup failed."), PSQLState.CONNECTION_UNABLE_TO_CONNECT);
 
-                int pid = pgStream.ReceiveIntegerR(4);
-                int ckey = pgStream.ReceiveIntegerR(4);
+                int pid = pgStream.ReceiveInteger4();
+                int ckey = pgStream.ReceiveInteger4();
 
                 if (logger.logDebug())
                     logger.debug(" <=BE BackendKeyData(pid=" + pid + ",ckey=" + ckey + ")");
@@ -434,7 +455,7 @@ public class ConnectionFactoryImpl extends ConnectionFactory {
 
             case 'E':
                 // Error
-                int l_elen = pgStream.ReceiveIntegerR(4);
+                int l_elen = pgStream.ReceiveInteger4();
                 ServerErrorMessage l_errorMsg = new ServerErrorMessage(pgStream.ReceiveString(l_elen - 4), logger.getLogLevel());
 
                 if (logger.logDebug())
@@ -444,7 +465,7 @@ public class ConnectionFactoryImpl extends ConnectionFactory {
 
             case 'N':
                 // Warning
-                int l_nlen = pgStream.ReceiveIntegerR(4);
+                int l_nlen = pgStream.ReceiveInteger4();
                 ServerErrorMessage l_warnMsg = new ServerErrorMessage(pgStream.ReceiveString(l_nlen - 4), logger.getLogLevel());
 
                 if (logger.logDebug())
@@ -455,7 +476,7 @@ public class ConnectionFactoryImpl extends ConnectionFactory {
 
             case 'S':
                 // ParameterStatus
-                int l_len = pgStream.ReceiveIntegerR(4);
+                int l_len = pgStream.ReceiveInteger4();
                 String name = pgStream.ReceiveString();
                 String value = pgStream.ReceiveString();
 
