@@ -1,10 +1,10 @@
 /*-------------------------------------------------------------------------
 *
-* Copyright (c) 2004-2005, PostgreSQL Global Development Group
+* Copyright (c) 2004-2008, PostgreSQL Global Development Group
 * Copyright (c) 2004, Open Cloud Limited.
 *
 * IDENTIFICATION
-*   $PostgreSQL: pgjdbc/org/postgresql/core/v3/SimpleQuery.java,v 1.11 2006/08/06 18:02:45 jurka Exp $
+*   $PostgreSQL: pgjdbc/org/postgresql/core/v3/SimpleQuery.java,v 1.14 2009/04/20 21:44:09 jurka Exp $
 *
 *-------------------------------------------------------------------------
 */
@@ -22,15 +22,18 @@ import java.lang.ref.PhantomReference;
  * @author Oliver Jowett (oliver@opencloud.com)
  */
 class SimpleQuery implements V3Query {
-    SimpleQuery(String[] fragments) {
+
+    SimpleQuery(String[] fragments, ProtocolConnectionImpl protoConnection)
+    {
         this.fragments = fragments;
+        this.protoConnection = protoConnection;
     }
 
     public ParameterList createParameterList() {
         if (fragments.length == 1)
             return NO_PARAMETERS;
 
-        return new SimpleParameterList(fragments.length - 1);
+        return new SimpleParameterList(fragments.length - 1, protoConnection);
     }
 
     public String toString(ParameterList parameters) {
@@ -81,6 +84,10 @@ class SimpleQuery implements V3Query {
         this.preparedTypes = paramTypes;
     }
 
+    int[] getStatementTypes() {
+        return preparedTypes;
+    }
+
     String getStatementName() {
         return statementName;
     }
@@ -91,16 +98,52 @@ class SimpleQuery implements V3Query {
 
         // Check for compatible types.
         for (int i = 0; i < paramTypes.length; ++i)
-            if (paramTypes[i] != 0 && paramTypes[i] != preparedTypes[i])
+            if (paramTypes[i] != Oid.UNSPECIFIED && paramTypes[i] != preparedTypes[i])
                 return false;
 
         return true;
     }
 
+    boolean hasUnresolvedTypes() {
+        if (preparedTypes == null)
+            return true;
+
+        for (int i=0; i<preparedTypes.length; i++) {
+            if (preparedTypes[i] == Oid.UNSPECIFIED)
+                return true;
+        }
+
+        return false;
+    }
+
     byte[] getEncodedStatementName() {
         return encodedStatementName;
     }
 
+    void setFields(Field[] fields) {
+        this.fields = fields;
+    }
+    Field[] getFields() {
+        return fields;
+    }
+
+    // Have we sent a Describe Portal message for this query yet?
+    boolean isPortalDescribed() {
+        return portalDescribed;
+    }
+    void setPortalDescribed(boolean portalDescribed) {
+        this.portalDescribed = portalDescribed;
+    }
+
+    // Have we sent a Describe Statement message for this query yet?
+    // Note that we might not have need to, so this may always be false.
+    boolean isStatementDescribed() {
+        return statementDescribed;
+    }
+    void setStatementDescribed(boolean statementDescribed) {
+        this.statementDescribed = statementDescribed;
+    }
+
     void setCleanupRef(PhantomReference cleanupRef) {
         if (this.cleanupRef != null) {
             this.cleanupRef.clear();
@@ -119,15 +162,22 @@ class SimpleQuery implements V3Query {
 
         statementName = null;
         encodedStatementName = null;
+        fields = null;
+        portalDescribed = false;
+        statementDescribed = false;
     }
 
     private final String[] fragments;
+    private final ProtocolConnectionImpl protoConnection;
     private String statementName;
     private byte[] encodedStatementName;
+    private Field[] fields;
+    private boolean portalDescribed;
+    private boolean statementDescribed;
     private PhantomReference cleanupRef;
     private int[] preparedTypes;
 
-    final static SimpleParameterList NO_PARAMETERS = new SimpleParameterList(0);
+    final static SimpleParameterList NO_PARAMETERS = new SimpleParameterList(0, null);
 }
 
 
