@@ -1,9 +1,9 @@
 /*-------------------------------------------------------------------------
 *
-* Copyright (c) 2004-2005, PostgreSQL Global Development Group
+* Copyright (c) 2004-2008, PostgreSQL Global Development Group
 *
 * IDENTIFICATION
-*   $PostgreSQL: pgjdbc/org/postgresql/jdbc2/AbstractJdbc2Connection.java,v 1.40 2006/12/01 08:53:45 jurka Exp $
+*   $PostgreSQL: pgjdbc/org/postgresql/jdbc2/AbstractJdbc2Connection.java,v 1.52 2009/07/01 05:00:40 jurka Exp $
 *
 *-------------------------------------------------------------------------
 */
@@ -15,7 +15,6 @@ import java.sql.*;
 import java.util.*;
 import org.postgresql.core.*;
 import org.postgresql.Driver;
-import org.postgresql.PGConnection;
 import org.postgresql.PGNotification;
 import org.postgresql.fastpath.Fastpath;
 import org.postgresql.largeobject.LargeObjectManager;
@@ -23,6 +22,7 @@ import org.postgresql.util.PSQLState;
 import org.postgresql.util.PGobject;
 import org.postgresql.util.PSQLException;
 import org.postgresql.util.GT;
+import org.postgresql.copy.*;
 
 /**
  * This class defines methods of the jdbc2 specification.
@@ -59,7 +59,7 @@ public abstract class AbstractJdbc2Connection implements BaseConnection
     /* Query that runs ROLLBACK */
     private final Query rollbackQuery;
 
-    private TypeInfoCache _typeCache;
+    private TypeInfo _typeCache;
 
     // Default statement prepare threshold.
     protected int prepareThreshold;
@@ -151,14 +151,24 @@ public abstract class AbstractJdbc2Connection implements BaseConnection
         }
 
         // Initialize timestamp stuff
-        timestampUtils = new TimestampUtils(haveMinimumServerVersion("7.4"));
+        timestampUtils = new TimestampUtils(haveMinimumServerVersion("7.4"), haveMinimumServerVersion("8.2"));
 
         // Initialize common queries.
         commitQuery = getQueryExecutor().createSimpleQuery("COMMIT");
         rollbackQuery = getQueryExecutor().createSimpleQuery("ROLLBACK");
 
+        int unknownLength = Integer.MAX_VALUE;
+        String strLength = info.getProperty("unknownLength");
+        if (strLength != null) {
+            try {
+                unknownLength = Integer.parseInt(strLength);
+            } catch (NumberFormatException nfe) {
+                throw new PSQLException(GT.tr("unknownLength parameter value must be an integer"), PSQLState.INVALID_PARAMETER_VALUE, nfe);
+            }
+        }
+
         // Initialize object handling
-        _typeCache = new TypeInfoCache(this);
+        _typeCache = createTypeInfo(this, unknownLength);
         initObjectTypes(info);
 
         if (Boolean.valueOf(info.getProperty("logUnclosedConnections")).booleanValue()) {
@@ -394,13 +404,11 @@ public abstract class AbstractJdbc2Connection implements BaseConnection
     {
         if (typemap != null)
         {
-            SQLData d = (SQLData) typemap.get(type);
-            if (d != null)
+            Class c = (Class) typemap.get(type);
+            if (c != null)
             {
                 // Handle the type (requires SQLInput & SQLOutput classes to be implemented)
-                if (logger.logDebug())
-                    logger.debug("getObject(String,String) with custom typemap");
-                throw org.postgresql.Driver.notImplemented(this.getClass(), "getObject(String,String)");
+                throw new PSQLException(GT.tr("Custom type maps are not supported."), PSQLState.NOT_IMPLEMENTED);
             }
         }
 
@@ -447,6 +455,16 @@ public abstract class AbstractJdbc2Connection implements BaseConnection
         }
     }
 
+    protected TypeInfo createTypeInfo(BaseConnection conn, int unknownLength)
+    {
+        return new TypeInfoCache(conn, unknownLength);
+    }
+
+    public TypeInfo getTypeInfo()
+    {
+        return _typeCache;
+    }
+
     public void addDataType(String type, String name)
     {
         try
@@ -733,7 +751,7 @@ public abstract class AbstractJdbc2Connection implements BaseConnection
         if (level == null)
             return Connection.TRANSACTION_READ_COMMITTED; // Best guess.
 
-        level = level.toUpperCase();
+        level = level.toUpperCase(Locale.US);
         if (level.indexOf("READ COMMITTED") != -1)
             return Connection.TRANSACTION_READ_COMMITTED;
         if (level.indexOf("READ UNCOMMITTED") != -1)
@@ -939,7 +957,7 @@ public abstract class AbstractJdbc2Connection implements BaseConnection
     }
 
     public String escapeString(String str) throws SQLException {
-        return Utils.appendEscapedString(null, str,
+        return Utils.appendEscapedLiteral(null, str,
                 protoConnection.getStandardConformingStrings()).toString();
     }
 
@@ -947,49 +965,6 @@ public abstract class AbstractJdbc2Connection implements BaseConnection
         return protoConnection.getStandardConformingStrings();
     }
 
-    /*
-     * This returns the java.sql.Types type for a PG type oid
-     *
-     * @param oid PostgreSQL type oid
-     * @return the java.sql.Types type
-     * @exception SQLException if a database access error occurs
-     */
-    public int getSQLType(int oid) throws SQLException
-    {
-        return _typeCache.getSQLType(oid);
-    }
-
-    public Iterator getPGTypeNamesWithSQLTypes()
-    {
-        return _typeCache.getPGTypeNamesWithSQLTypes();
-    }
-
-    /*
-     * This returns the oid for a given PG data type
-     * @param typeName PostgreSQL type name
-     * @return PostgreSQL oid value for a field of this type, or 0 if not found
-     */
-    public int getPGType(String typeName) throws SQLException
-    {
-        return _typeCache.getPGType(typeName);
-    }
-
-    public String getJavaClass(int oid) throws SQLException
-    {
-        return _typeCache.getJavaClass(oid);
-    }
-
-    /*
-     * We also need to get the PG type name as returned by the back end.
-     *
-     * @return the String representation of the type, or null if not fould
-     * @exception SQLException if a database access error occurs
-     */
-    public String getPGType(int oid) throws SQLException
-    {
-        return _typeCache.getPGType(oid);
-    }
-
     // This is a cache of the DatabaseMetaData instance for this connection
     protected java.sql.DatabaseMetaData metadata;
 
@@ -1011,6 +986,7 @@ public abstract class AbstractJdbc2Connection implements BaseConnection
 
     public PGNotification[] getNotifications() throws SQLException
     {
+        getQueryExecutor().processNotifies();
         // Backwards-compatibility hand-holding.
         PGNotification[] notifications = protoConnection.getNotifications();
         return (notifications.length == 0 ? null : notifications);
@@ -1075,12 +1051,6 @@ public abstract class AbstractJdbc2Connection implements BaseConnection
         }
     }
 
-
-    public int getSQLType(String pgTypeName)
-    {
-        return _typeCache.getSQLType(pgTypeName);
-    }
-    
     public int getProtocolVersion()
     {
         return protoConnection.getProtocolVersion();
@@ -1090,4 +1060,12 @@ public abstract class AbstractJdbc2Connection implements BaseConnection
     {
         return bindStringAsVarchar;
     }
+
+    private CopyManager copyManager = null;
+    public CopyManager getCopyAPI() throws SQLException
+    {
+        if (copyManager == null)
+            copyManager = new CopyManager(this);
+        return copyManager;
+    }
 }
