@@ -1,9 +1,9 @@
 /*-------------------------------------------------------------------------
 *
-* Copyright (c) 2004-2005, PostgreSQL Global Development Group
+* Copyright (c) 2004-2008, PostgreSQL Global Development Group
 *
 * IDENTIFICATION
-*   $PostgreSQL: pgjdbc/org/postgresql/jdbc3/AbstractJdbc3Connection.java,v 1.17 2005/04/10 21:54:16 jurka Exp $
+*   $PostgreSQL: pgjdbc/org/postgresql/jdbc3/AbstractJdbc3Connection.java,v 1.20 2009/07/01 01:29:38 jurka Exp $
 *
 *-------------------------------------------------------------------------
 */
@@ -349,8 +349,14 @@ public abstract class AbstractJdbc3Connection extends org.postgresql.jdbc2.Abstr
     throws SQLException
     {
         if (autoGeneratedKeys != Statement.NO_GENERATED_KEYS)
-            throw new PSQLException(GT.tr("Returning autogenerated keys is not supported."), PSQLState.NOT_IMPLEMENTED);
-        return prepareStatement(sql);
+            sql = AbstractJdbc3Statement.addReturning(this, sql, new String[]{"*"}, false);
+
+        PreparedStatement ps = prepareStatement(sql);
+
+        if (autoGeneratedKeys != Statement.NO_GENERATED_KEYS)
+            ((AbstractJdbc3Statement)ps).wantsGeneratedKeysAlways = true;
+
+        return ps;
     }
 
 
@@ -396,9 +402,10 @@ public abstract class AbstractJdbc3Connection extends org.postgresql.jdbc2.Abstr
     public PreparedStatement prepareStatement(String sql, int columnIndexes[])
     throws SQLException
     {
-        if (columnIndexes.length != 0)
-            throw new PSQLException(GT.tr("Returning autogenerated keys is not supported."), PSQLState.NOT_IMPLEMENTED);
-        return prepareStatement(sql);
+        if (columnIndexes == null || columnIndexes.length == 0)
+            return prepareStatement(sql);
+        
+        throw new PSQLException(GT.tr("Returning autogenerated keys is not supported."), PSQLState.NOT_IMPLEMENTED);
     }
 
 
@@ -444,9 +451,15 @@ public abstract class AbstractJdbc3Connection extends org.postgresql.jdbc2.Abstr
     public PreparedStatement prepareStatement(String sql, String columnNames[])
     throws SQLException
     {
-        if (columnNames.length != 0)
-            throw new PSQLException(GT.tr("Returning autogenerated keys is not supported."), PSQLState.NOT_IMPLEMENTED);
-        return prepareStatement(sql);
+        if (columnNames != null && columnNames.length != 0)
+            sql = AbstractJdbc3Statement.addReturning(this, sql, columnNames, true);
+
+        PreparedStatement ps = prepareStatement(sql);
+
+        if (columnNames != null && columnNames.length != 0)
+            ((AbstractJdbc3Statement)ps).wantsGeneratedKeysAlways = true;
+
+        return ps;
     }
 
 }
