@@ -1,9 +1,9 @@
 /*-------------------------------------------------------------------------
 *
-* Copyright (c) 2004-2005, PostgreSQL Global Development Group
+* Copyright (c) 2004-2008, PostgreSQL Global Development Group
 *
 * IDENTIFICATION
-*   $PostgreSQL: pgjdbc/org/postgresql/jdbc3/AbstractJdbc3Statement.java,v 1.21 2006/06/08 10:34:50 jurka Exp $
+*   $PostgreSQL: pgjdbc/org/postgresql/jdbc3/AbstractJdbc3Statement.java,v 1.24 2009/01/28 09:50:21 jurka Exp $
 *
 *-------------------------------------------------------------------------
 */
@@ -16,6 +16,7 @@ import java.util.Vector;
 
 import org.postgresql.util.PSQLException;
 import org.postgresql.util.PSQLState;
+import org.postgresql.core.Utils;
 import org.postgresql.core.QueryExecutor;
 import org.postgresql.core.Field;
 import org.postgresql.core.BaseConnection;
@@ -106,7 +107,11 @@ public abstract class AbstractJdbc3Statement extends org.postgresql.jdbc2.Abstra
      */
     public ResultSet getGeneratedKeys() throws SQLException
     {
-        return createDriverResultSet(new Field[0], new Vector());
+        checkClosed();
+        if (generatedKeys == null || generatedKeys.getResultSet() == null)
+            return createDriverResultSet(new Field[0], new Vector());
+
+        return generatedKeys.getResultSet();
     }
 
     /**
@@ -136,7 +141,38 @@ public abstract class AbstractJdbc3Statement extends org.postgresql.jdbc2.Abstra
         if (autoGeneratedKeys == Statement.NO_GENERATED_KEYS)
             return executeUpdate(sql);
 
-        throw new PSQLException(GT.tr("Returning autogenerated keys is not supported."), PSQLState.NOT_IMPLEMENTED);
+        sql = addReturning(connection, sql, new String[]{"*"}, false);
+        wantsGeneratedKeysOnce = true;
+
+        return executeUpdate(sql);
+    }
+
+    static String addReturning(BaseConnection connection, String sql, String columns[], boolean escape) throws SQLException
+    {
+        if (!connection.haveMinimumServerVersion("8.2"))
+            throw new PSQLException(GT.tr("Returning autogenerated keys is only supported for 8.2 and later servers."), PSQLState.NOT_IMPLEMENTED);
+
+        sql = sql.trim();
+        if (sql.endsWith(";"))
+            sql = sql.substring(0, sql.length()-1);
+
+        StringBuffer sb = new StringBuffer(sql);
+        sb.append(" RETURNING ");
+        for (int i=0; i<columns.length; i++) {
+            if (i != 0)
+                sb.append(", ");
+            // If given user provided column names, quote and escape them.
+            // This isn't likely to be popular as it enforces case sensitivity,
+            // but it does match up with our handling of things like
+            // DatabaseMetaData.getColumns and is necessary for the same
+            // reasons.
+            if (escape)
+                Utils.appendEscapedIdentifier(sb, columns[i]);
+            else
+                sb.append(columns[i]);
+        }
+
+        return sb.toString();
     }
 
     /**
@@ -159,10 +195,10 @@ public abstract class AbstractJdbc3Statement extends org.postgresql.jdbc2.Abstra
      */
     public int executeUpdate(String sql, int columnIndexes[]) throws SQLException
     {
-        if (columnIndexes.length == 0)
+        if (columnIndexes == null || columnIndexes.length == 0)
             return executeUpdate(sql);
 
-        throw new PSQLException(GT.tr("Returning autogenerated keys is not supported."), PSQLState.NOT_IMPLEMENTED);
+        throw new PSQLException(GT.tr("Returning autogenerated keys by column index is not supported."), PSQLState.NOT_IMPLEMENTED);
     }
 
     /**
@@ -184,10 +220,13 @@ public abstract class AbstractJdbc3Statement extends org.postgresql.jdbc2.Abstra
      */
     public int executeUpdate(String sql, String columnNames[]) throws SQLException
     {
-        if (columnNames.length == 0)
+        if (columnNames == null || columnNames.length == 0)
             return executeUpdate(sql);
 
-        throw new PSQLException(GT.tr("Returning autogenerated keys is not supported."), PSQLState.NOT_IMPLEMENTED);
+        sql = addReturning(connection, sql, columnNames, true);
+        wantsGeneratedKeysOnce = true;
+
+        return executeUpdate(sql);
     }
 
     /**
@@ -231,7 +270,10 @@ public abstract class AbstractJdbc3Statement extends org.postgresql.jdbc2.Abstra
         if (autoGeneratedKeys == Statement.NO_GENERATED_KEYS)
             return execute(sql);
 
-        throw new PSQLException(GT.tr("Returning autogenerated keys is not supported."), PSQLState.NOT_IMPLEMENTED);
+        sql = addReturning(connection, sql, new String[]{"*"}, false);
+        wantsGeneratedKeysOnce = true;
+
+        return execute(sql);
     }
 
     /**
@@ -271,10 +313,10 @@ public abstract class AbstractJdbc3Statement extends org.postgresql.jdbc2.Abstra
      */
     public boolean execute(String sql, int columnIndexes[]) throws SQLException
     {
-        if (columnIndexes.length == 0)
+        if (columnIndexes == null || columnIndexes.length == 0)
             return execute(sql);
 
-        throw new PSQLException(GT.tr("Returning autogenerated keys is not supported."), PSQLState.NOT_IMPLEMENTED);
+        throw new PSQLException(GT.tr("Returning autogenerated keys by column index is not supported."), PSQLState.NOT_IMPLEMENTED);
     }
 
     /**
@@ -315,10 +357,13 @@ public abstract class AbstractJdbc3Statement extends org.postgresql.jdbc2.Abstra
      */
     public boolean execute(String sql, String columnNames[]) throws SQLException
     {
-        if (columnNames.length == 0)
+        if (columnNames == null || columnNames.length == 0)
             return execute(sql);
 
-        throw new PSQLException(GT.tr("Returning autogenerated keys is not supported."), PSQLState.NOT_IMPLEMENTED);
+        sql = addReturning(connection, sql, columnNames, true);
+        wantsGeneratedKeysOnce = true;
+
+        return execute(sql);
     }
 
     /**
