@@ -1,9 +1,9 @@
 /*-------------------------------------------------------------------------
 *
-* Copyright (c) 2004-2005, PostgreSQL Global Development Group
+* Copyright (c) 2004-2008, PostgreSQL Global Development Group
 *
 * IDENTIFICATION
-*   $PostgreSQL: pgjdbc/org/postgresql/test/jdbc2/BlobTest.java,v 1.19 2006/10/31 06:12:47 jurka Exp $
+*   $PostgreSQL: pgjdbc/org/postgresql/test/jdbc2/BlobTest.java,v 1.22 2008/01/08 06:56:30 jurka Exp $
 *
 *-------------------------------------------------------------------------
 */
@@ -153,18 +153,66 @@ public class BlobTest extends TestCase
         assertEquals(data[3], 'l');
     }
 
+    public void testMultipleStreams() throws Exception
+    {
+        assertTrue(uploadFile("build.xml", NATIVE_STREAM) > 0);
+
+        Statement stmt = con.createStatement();
+        ResultSet rs = stmt.executeQuery("SELECT lo FROM testblob");
+        assertTrue(rs.next());
+
+        Blob lob = rs.getBlob(1);
+        byte data[] = new byte[2];
+
+        InputStream is = lob.getBinaryStream();
+        assertEquals(data.length, is.read(data));
+        assertEquals(data[0], '<');
+        assertEquals(data[1], '?');
+        is.close();
+
+        is = lob.getBinaryStream();
+        assertEquals(data.length, is.read(data));
+        assertEquals(data[0], '<');
+        assertEquals(data[1], '?');
+        is.close();
+    }
+
+    public void testParallelStreams() throws Exception
+    {
+        assertTrue(uploadFile("build.xml", NATIVE_STREAM) > 0);
+
+        Statement stmt = con.createStatement();
+        ResultSet rs = stmt.executeQuery("SELECT lo FROM testblob");
+        assertTrue(rs.next());
+
+        Blob lob = rs.getBlob(1);
+        InputStream is1 = lob.getBinaryStream();
+        InputStream is2 = lob.getBinaryStream();
+
+        while (true) {
+            int i1 = is1.read();
+            int i2 = is2.read();
+            assertEquals(i1, i2);
+            if (i1 == -1)
+                break;
+        }
+
+        is1.close();
+        is2.close();
+    }
+
     /*
      * Helper - uploads a file into a blob using old style methods. We use this
      * because it always works, and we can use it as a base to test the new
      * methods.
      */
-    private int uploadFile(String file, int method) throws Exception
+    private long uploadFile(String file, int method) throws Exception
     {
         LargeObjectManager lom = ((org.postgresql.PGConnection)con).getLargeObjectAPI();
 
         FileInputStream fis = new FileInputStream(file);
 
-        int oid = lom.create(LargeObjectManager.READWRITE);
+        long oid = lom.createLO(LargeObjectManager.READWRITE);
         LargeObject blob = lom.open(oid);
 
         int s, t;
@@ -227,7 +275,7 @@ public class BlobTest extends TestCase
         while (rs.next())
         {
             String file = rs.getString(1);
-            int oid = rs.getInt(2);
+            long oid = rs.getLong(2);
 
             FileInputStream fis = new FileInputStream(file);
             LargeObject blob = lom.open(oid);
