@@ -1,9 +1,9 @@
 /*-------------------------------------------------------------------------
 *
-* Copyright (c) 2004-2005, PostgreSQL Global Development Group
+* Copyright (c) 2004-2008, PostgreSQL Global Development Group
 *
 * IDENTIFICATION
-*   $PostgreSQL: pgjdbc/org/postgresql/test/jdbc2/DatabaseMetaDataTest.java,v 1.36 2006/02/03 21:10:15 jurka Exp $
+*   $PostgreSQL: pgjdbc/org/postgresql/test/jdbc2/DatabaseMetaDataTest.java,v 1.44 2008/11/07 09:11:37 jurka Exp $
 *
 *-------------------------------------------------------------------------
 */
@@ -33,27 +33,53 @@ public class DatabaseMetaDataTest extends TestCase
     protected void setUp() throws Exception
     {
         con = TestUtil.openDB();
-        TestUtil.createTable( con, "testmetadata", "id int4, name text, updated timestamp, colour text, quest text" );
+        TestUtil.createTable( con, "testmetadata", "id int4, name text, updated timestamptz, colour text, quest text" );
         TestUtil.dropSequence( con, "sercoltest_b_seq");
         TestUtil.dropSequence( con, "sercoltest_c_seq");
         TestUtil.createTable( con, "sercoltest", "a int, b serial, c bigserial");
         TestUtil.createTable( con, "\"a\\\"", "a int4");
         TestUtil.createTable( con, "\"a'\"", "a int4");
+        TestUtil.createTable( con, "arraytable", "a numeric(5,2)[], b varchar(100)[]");
 
         Statement stmt = con.createStatement();
         //we add the following comments to ensure the joins to the comments
         //are done correctly. This ensures we correctly test that case.
         stmt.execute("comment on table testmetadata is 'this is a table comment'");
         stmt.execute("comment on column testmetadata.id is 'this is a column comment'");
+
+        stmt.execute("CREATE OR REPLACE FUNCTION f1(int, varchar) RETURNS int AS 'SELECT 1;' LANGUAGE 'SQL'");
+        if (TestUtil.haveMinimumServerVersion(con, "8.0")) {
+            stmt.execute("CREATE OR REPLACE FUNCTION f2(a int, b varchar) RETURNS int AS 'SELECT 1;' LANGUAGE 'SQL'");
+        }
+        if (TestUtil.haveMinimumServerVersion(con, "8.1")) {
+            stmt.execute("CREATE OR REPLACE FUNCTION f3(IN a int, INOUT b varchar, OUT c timestamptz) AS $f$ BEGIN b := 'a'; c := now(); return; END; $f$ LANGUAGE 'plpgsql'");
+        }
+        stmt.execute("CREATE OR REPLACE FUNCTION f4(int) RETURNS testmetadata AS 'SELECT 1, ''a''::text, now(), ''c''::text, ''q''::text' LANGUAGE 'SQL'");
+        stmt.close();
     }
+
     protected void tearDown() throws Exception
     {
+        // Drop function first because it depends on the
+        // testmetadata table's type
+        Statement stmt = con.createStatement();
+        stmt.execute("DROP FUNCTION f4(int)");
+
         TestUtil.dropTable( con, "testmetadata" );
         TestUtil.dropTable( con, "sercoltest");
         TestUtil.dropSequence( con, "sercoltest_b_seq");
         TestUtil.dropSequence( con, "sercoltest_c_seq");
         TestUtil.dropTable( con, "\"a\\\"");
         TestUtil.dropTable( con, "\"a'\"");
+        TestUtil.dropTable( con, "arraytable");
+
+        stmt.execute("DROP FUNCTION f1(int, varchar)");
+        if (TestUtil.haveMinimumServerVersion(con, "8.0")) {
+            stmt.execute("DROP FUNCTION f2(int, varchar)");
+        }
+        if (TestUtil.haveMinimumServerVersion(con, "8.1")) {
+            stmt.execute("DROP FUNCTION f3(int, varchar)");
+        }
 
         TestUtil.closeDB( con );
     }
@@ -143,6 +169,7 @@ public class DatabaseMetaDataTest extends TestCase
 
         TestUtil.dropTable( con1, "vv" );
         TestUtil.dropTable( con1, "ww" );
+        TestUtil.closeDB(con1);
     }
 
     public void testForeignKeyActions() throws Exception
@@ -168,6 +195,7 @@ public class DatabaseMetaDataTest extends TestCase
         TestUtil.dropTable(conn, "fkt2");
         TestUtil.dropTable(conn, "fkt1");
         TestUtil.dropTable(conn, "pkt");
+        TestUtil.closeDB(conn);
     }
 
     public void testForeignKeysToUniqueIndexes() throws Exception
@@ -286,7 +314,7 @@ public class DatabaseMetaDataTest extends TestCase
         TestUtil.dropTable( con1, "users" );
         TestUtil.dropTable( con1, "people" );
         TestUtil.dropTable( con1, "policy" );
-
+        TestUtil.closeDB(con1);
     }
 
     public void testColumns() throws SQLException
@@ -365,6 +393,7 @@ public class DatabaseMetaDataTest extends TestCase
         Statement stmt = con.createStatement();
         stmt.execute("create index idx_id on testmetadata (id)");
         stmt.execute("create index idx_func_single on testmetadata (upper(colour))");
+        stmt.execute("create unique index idx_un_id on testmetadata(id)");
         if (TestUtil.haveMinimumServerVersion(con, "7.4")) {
             stmt.execute("create index idx_func_multi on testmetadata (upper(colour), upper(quest))");
             stmt.execute("create index idx_func_mixed on testmetadata (colour, upper(quest))");
@@ -374,6 +403,12 @@ public class DatabaseMetaDataTest extends TestCase
         assertNotNull(dbmd);
         ResultSet rs = dbmd.getIndexInfo(null, null, "testmetadata", false, false);
 
+        assertTrue(rs.next());
+        assertEquals("idx_un_id", rs.getString("INDEX_NAME"));
+        assertEquals(1, rs.getInt("ORDINAL_POSITION"));
+        assertEquals("id", rs.getString("COLUMN_NAME"));
+        assertTrue(!rs.getBoolean("NON_UNIQUE"));
+
         if (TestUtil.haveMinimumServerVersion(con, "7.4")) {
             assertTrue(rs.next());
             assertEquals("idx_func_mixed", rs.getString("INDEX_NAME"));
@@ -405,6 +440,7 @@ public class DatabaseMetaDataTest extends TestCase
         assertEquals("idx_id", rs.getString("INDEX_NAME"));
         assertEquals(1, rs.getInt("ORDINAL_POSITION"));
         assertEquals("id", rs.getString("COLUMN_NAME"));
+        assertTrue(rs.getBoolean("NON_UNIQUE"));
 
         assertTrue(!rs.next());
 
@@ -420,12 +456,114 @@ public class DatabaseMetaDataTest extends TestCase
         rs.close();
     }
 
-    public void testProcedureColumns() throws SQLException
+    public void testFuncWithoutNames() throws SQLException
     {
-        // At the moment just test that no exceptions are thrown KJ
         DatabaseMetaData dbmd = con.getMetaData();
         assertNotNull(dbmd);
-        ResultSet rs = dbmd.getProcedureColumns(null, null, null, null);
+        ResultSet rs = dbmd.getProcedureColumns(null, null, "f1", null);
+
+        assertTrue(rs.next());
+        assertEquals("returnValue", rs.getString(4));
+        assertEquals(DatabaseMetaData.procedureColumnReturn, rs.getInt(5));
+
+        assertTrue(rs.next());
+        assertEquals("$1", rs.getString(4));
+        assertEquals(DatabaseMetaData.procedureColumnIn, rs.getInt(5));
+        assertEquals(Types.INTEGER, rs.getInt(6));
+
+        assertTrue(rs.next());
+        assertEquals("$2", rs.getString(4));
+        assertEquals(DatabaseMetaData.procedureColumnIn, rs.getInt(5));
+        assertEquals(Types.VARCHAR, rs.getInt(6));
+
+        assertTrue(!rs.next());
+
+        rs.close();
+    }
+
+    public void testFuncWithNames() throws SQLException
+    {
+        if (!TestUtil.haveMinimumServerVersion(con, "8.0"))
+            return;
+
+        DatabaseMetaData dbmd = con.getMetaData();
+        ResultSet rs = dbmd.getProcedureColumns(null, null, "f2", null);
+
+        assertTrue(rs.next());
+
+        assertTrue(rs.next());
+        assertEquals("a", rs.getString(4));
+
+        assertTrue(rs.next());
+        assertEquals("b", rs.getString(4));
+
+        assertTrue(!rs.next());
+        
+        rs.close();
+    }
+
+    public void testFuncWithDirection() throws SQLException
+    {
+        if (!TestUtil.haveMinimumServerVersion(con, "8.1"))
+            return;
+
+        DatabaseMetaData dbmd = con.getMetaData();
+        ResultSet rs = dbmd.getProcedureColumns(null, null, "f3", null);
+
+        assertTrue(rs.next());
+        assertEquals("a", rs.getString(4));
+        assertEquals(DatabaseMetaData.procedureColumnIn, rs.getInt(5));
+        assertEquals(Types.INTEGER, rs.getInt(6));
+
+        assertTrue(rs.next());
+        assertEquals("b", rs.getString(4));
+        assertEquals(DatabaseMetaData.procedureColumnInOut, rs.getInt(5));
+        assertEquals(Types.VARCHAR, rs.getInt(6));
+
+        assertTrue(rs.next());
+        assertEquals("c", rs.getString(4));
+        assertEquals(DatabaseMetaData.procedureColumnOut, rs.getInt(5));
+        assertEquals(Types.TIMESTAMP, rs.getInt(6));
+
+        rs.close();
+    }
+
+    public void testFuncReturningComposite() throws SQLException
+    {
+        DatabaseMetaData dbmd = con.getMetaData();
+        ResultSet rs = dbmd.getProcedureColumns(null, null, "f4", null);
+
+        assertTrue(rs.next());
+        assertEquals("$1", rs.getString(4));
+        assertEquals(DatabaseMetaData.procedureColumnIn, rs.getInt(5));
+        assertEquals(Types.INTEGER, rs.getInt(6));
+
+        assertTrue(rs.next());
+        assertEquals("id", rs.getString(4));
+        assertEquals(DatabaseMetaData.procedureColumnResult, rs.getInt(5));
+        assertEquals(Types.INTEGER, rs.getInt(6));
+
+        assertTrue(rs.next());
+        assertEquals("name", rs.getString(4));
+        assertEquals(DatabaseMetaData.procedureColumnResult, rs.getInt(5));
+        assertEquals(Types.VARCHAR, rs.getInt(6));
+
+        assertTrue(rs.next());
+        assertEquals("updated", rs.getString(4));
+        assertEquals(DatabaseMetaData.procedureColumnResult, rs.getInt(5));
+        assertEquals(Types.TIMESTAMP, rs.getInt(6));
+
+        assertTrue(rs.next());
+        assertEquals("colour", rs.getString(4));
+        assertEquals(DatabaseMetaData.procedureColumnResult, rs.getInt(5));
+        assertEquals(Types.VARCHAR, rs.getInt(6));
+
+        assertTrue(rs.next());
+        assertEquals("quest", rs.getString(4));
+        assertEquals(DatabaseMetaData.procedureColumnResult, rs.getInt(5));
+        assertEquals(Types.VARCHAR, rs.getInt(6));
+
+        assertTrue(!rs.next());
         rs.close();
     }
 
@@ -524,12 +662,16 @@ public class DatabaseMetaDataTest extends TestCase
 
     public void testSearchStringEscape() throws Exception {
         DatabaseMetaData dbmd = con.getMetaData();
-        Statement stmt = con.createStatement();
-        ResultSet rs = stmt.executeQuery("SELECT 'a' LIKE '" + dbmd.getSearchStringEscape() + "_'");
+        String pattern = dbmd.getSearchStringEscape() + "_";
+        PreparedStatement pstmt = con.prepareStatement("SELECT 'a' LIKE ?, '_' LIKE ?");
+        pstmt.setString(1, pattern);
+        pstmt.setString(2, pattern);
+        ResultSet rs = pstmt.executeQuery();
         assertTrue (rs.next());
         assertTrue(!rs.getBoolean(1));
+        assertTrue(rs.getBoolean(2));
         rs.close();
-        stmt.close();
+        pstmt.close();
     }
 
     public void testGetUDTQualified() throws Exception
@@ -766,4 +908,48 @@ public class DatabaseMetaDataTest extends TestCase
             }
         }
     }
+
+    public void testTypeInfoSigned() throws SQLException
+    {
+        DatabaseMetaData dbmd = con.getMetaData();
+        ResultSet rs = dbmd.getTypeInfo();
+        while (rs.next()) {
+            if ("int4".equals(rs.getString("TYPE_NAME"))) {
+                assertEquals(false, rs.getBoolean("UNSIGNED_ATTRIBUTE"));
+            } else if ("float8".equals(rs.getString("TYPE_NAME"))) {
+                assertEquals(false, rs.getBoolean("UNSIGNED_ATTRIBUTE"));
+            } else if ("text".equals(rs.getString("TYPE_NAME"))) {
+                assertEquals(true, rs.getBoolean("UNSIGNED_ATTRIBUTE"));
+            }
+        }
+    }
+
+    public void testTypeInfoQuoting() throws SQLException
+    {
+        DatabaseMetaData dbmd = con.getMetaData();
+        ResultSet rs = dbmd.getTypeInfo();
+        while (rs.next()) {
+            if ("int4".equals(rs.getString("TYPE_NAME"))) {
+                assertNull(rs.getString("LITERAL_PREFIX"));
+            } else if ("text".equals(rs.getString("TYPE_NAME"))) {
+                assertEquals("'", rs.getString("LITERAL_PREFIX"));
+                assertEquals("'", rs.getString("LITERAL_SUFFIX"));
+            }
+        }
+    }
+
+    public void testInformationAboutArrayTypes() throws SQLException
+    {
+        DatabaseMetaData dbmd = con.getMetaData();
+        ResultSet rs = dbmd.getColumns("", "", "arraytable", "");
+        assertTrue(rs.next());
+        assertEquals("a", rs.getString("COLUMN_NAME"));
+        assertEquals(5, rs.getInt("COLUMN_SIZE"));
+        assertEquals(2, rs.getInt("DECIMAL_DIGITS"));
+	assertTrue(rs.next());
+        assertEquals("b", rs.getString("COLUMN_NAME"));
+        assertEquals(100, rs.getInt("COLUMN_SIZE"));
+        assertTrue(!rs.next());
+    }
+
 }
