@@ -1,9 +1,9 @@
 /*-------------------------------------------------------------------------
 *
-* Copyright (c) 2004-2005, PostgreSQL Global Development Group
+* Copyright (c) 2004-2008, PostgreSQL Global Development Group
 *
 * IDENTIFICATION
-*   $PostgreSQL: pgjdbc/org/postgresql/test/jdbc2/ResultSetTest.java,v 1.27 2005/11/24 02:31:43 oliver Exp $
+*   $PostgreSQL: pgjdbc/org/postgresql/test/jdbc2/ResultSetTest.java,v 1.30 2008/02/19 06:12:24 jurka Exp $
 *
 *-------------------------------------------------------------------------
 */
@@ -14,6 +14,7 @@ import java.sql.Connection;
 import java.sql.ResultSet;
 import java.sql.Statement;
 import java.sql.SQLException;
+import java.util.Locale;
 
 import junit.framework.TestCase;
 
@@ -693,4 +694,35 @@ public class ResultSetTest extends TestCase
         try { rs.clearWarnings(); fail("Expected SQLException"); }
             catch (SQLException e) { }
     }
+
+    /*
+     * The JDBC spec says when you have duplicate column names,
+     * the first one should be returned.
+     */
+    public void testDuplicateColumnNameOrder() throws SQLException
+    {
+        Statement stmt = con.createStatement();
+        ResultSet rs = stmt.executeQuery("SELECT 1 AS a, 2 AS a");
+        assertTrue(rs.next());
+        assertEquals(1, rs.getInt("a"));
+    }
+
+    public void testTurkishLocale() throws SQLException
+    {
+        Locale current = Locale.getDefault();
+        try {
+            Locale.setDefault(new Locale("tr", "TR"));
+            Statement stmt = con.createStatement();
+            ResultSet rs = stmt.executeQuery("SELECT id FROM testrs");
+            int sum = 0;
+            while (rs.next()) {
+                sum += rs.getInt("ID");
+            }
+            rs.close();
+            assertEquals(25, sum);
+        } finally {
+            Locale.setDefault(current);
+        }
+    }
+
 }
