@@ -1,15 +1,14 @@
 /*-------------------------------------------------------------------------
 *
-* Copyright (c) 2004-2005, PostgreSQL Global Development Group
+* Copyright (c) 2004-2008, PostgreSQL Global Development Group
 *
 * IDENTIFICATION
-*   $PostgreSQL: pgjdbc/org/postgresql/test/jdbc2/StatementTest.java,v 1.24 2006/12/01 08:53:46 jurka Exp $
+*   $PostgreSQL: pgjdbc/org/postgresql/test/jdbc2/StatementTest.java,v 1.28 2008/11/16 12:14:07 jurka Exp $
 *
 *-------------------------------------------------------------------------
 */
 package org.postgresql.test.jdbc2;
 
-import org.postgresql.jdbc2.AbstractJdbc2Connection;
 import org.postgresql.test.TestUtil;
 import junit.framework.*;
 import java.sql.*;
@@ -232,13 +231,12 @@ public class StatementTest extends TestCase
         assertEquals(1, rs.getDouble(5), 0.00001);
 
         rs = stmt.executeQuery("select {fn pi()}, {fn power(7,-2.3)}," +
-            " {fn radians(-180)},{fn rand(-2.3)},{fn round(3.1294,2)}");
+            " {fn radians(-180)},{fn round(3.1294,2)}");
         assertTrue(rs.next());
         assertEquals(Math.PI, rs.getDouble(1), 0.00001);
         assertEquals(Math.pow(7,-2.3), rs.getDouble(2), 0.00001);
         assertEquals(-Math.PI, rs.getDouble(3), 0.00001);
-        rs.getDouble(4);  // for random all we can test is that it returns something
-        assertEquals(3.13, rs.getDouble(5), 0.00001);
+        assertEquals(3.13, rs.getDouble(4), 0.00001);
 
         rs = stmt.executeQuery("select {fn sign(-2.3)}, {fn sin(-2.3)}," +
             " {fn sqrt(2.3)},{fn tan(-2.3)},{fn truncate(3.1294,2)}");
@@ -288,6 +286,22 @@ public class StatementTest extends TestCase
         assertEquals("bc",rs.getString(6));
         assertEquals("ABCD",rs.getString(7));
     }
+
+    public void testDateFuncWithParam() throws SQLException
+    {
+        // Prior to 8.0 there is not an interval + timestamp operator,
+        // so timestampadd does not work.
+        //
+        if (!TestUtil.haveMinimumServerVersion(con, "8.0"))
+            return;
+
+        PreparedStatement ps = con.prepareStatement("SELECT {fn timestampadd(SQL_TSI_QUARTER, ? ,{fn now()})}, {fn timestampadd(SQL_TSI_MONTH, ?, {fn now()})} ");
+        ps.setInt(1, 4);
+        ps.setInt(2, 12);
+        ResultSet rs = ps.executeQuery();
+        assertTrue(rs.next());
+        assertEquals(rs.getTimestamp(1), rs.getTimestamp(2));
+    }
     
     public void testDateFunctions() throws SQLException
     {
@@ -304,7 +318,7 @@ public class StatementTest extends TestCase
         // ensure sunday =>1 and monday =>2
         assertEquals(2,rs.getInt(5));
 
-	// Prior to 8.0 there is not an interval + timestamp operator,
+        // Prior to 8.0 there is not an interval + timestamp operator,
         // so timestampadd does not work.
         //
         if (!TestUtil.haveMinimumServerVersion(con, "8.0"))
