@@ -1,9 +1,9 @@
 /*-------------------------------------------------------------------------
 *
-* Copyright (c) 2004-2005, PostgreSQL Global Development Group
+* Copyright (c) 2004-2008, PostgreSQL Global Development Group
 *
 * IDENTIFICATION
-*   $PostgreSQL: pgjdbc/org/postgresql/test/jdbc3/Jdbc3TestSuite.java,v 1.16 2006/05/15 09:35:57 jurka Exp $
+*   $PostgreSQL: pgjdbc/org/postgresql/test/jdbc3/Jdbc3TestSuite.java,v 1.19 2008/11/15 17:48:53 jurka Exp $
 *
 *-------------------------------------------------------------------------
 */
@@ -34,6 +34,10 @@ public class Jdbc3TestSuite extends TestSuite
             {
                 suite.addTestSuite(Jdbc3CallableStatementTest.class);
             }
+            if ( TestUtil.haveMinimumServerVersion( con, "8.2") )
+            {
+                suite.addTestSuite(GeneratedKeysTest.class);
+            }
             con.close();
         }
         catch (Exception ex )
@@ -46,6 +50,7 @@ public class Jdbc3TestSuite extends TestSuite
         suite.addTestSuite(ResultSetTest.class);
         suite.addTestSuite(ParameterMetaDataTest.class);
         suite.addTestSuite(Jdbc3BlobTest.class);
+        suite.addTestSuite(DatabaseMetaDataTest.class);
         return suite;
     }
 }
