@@ -1,15 +1,14 @@
 /*-------------------------------------------------------------------------
 *
-* Copyright (c) 2004-2005, PostgreSQL Global Development Group
+* Copyright (c) 2004-2008, PostgreSQL Global Development Group
 *
 * IDENTIFICATION
-*   $PostgreSQL: pgjdbc/org/postgresql/util/ServerErrorMessage.java,v 1.10 2005/11/24 02:29:23 oliver Exp $
+*   $PostgreSQL: pgjdbc/org/postgresql/util/ServerErrorMessage.java,v 1.13 2008/05/20 19:51:38 jurka Exp $
 *
 *-------------------------------------------------------------------------
 */
 package org.postgresql.util;
 
-import org.postgresql.Driver;
 import java.util.Hashtable;
 import java.io.Serializable;
 
@@ -154,18 +153,17 @@ public class ServerErrorMessage implements Serializable
         l_message = (String)m_mesgParts.get(DETAIL);
         if (l_message != null)
             l_totalMessage.append("\n  ").append(GT.tr("Detail: {0}", l_message));
-        if (verbosity > 0)
-        {
-            l_message = (String)m_mesgParts.get(HINT);
-            if (l_message != null)
-                l_totalMessage.append("\n  ").append(GT.tr("Hint: {0}", l_message));
-            l_message = (String)m_mesgParts.get(POSITION);
-            if (l_message != null)
-                l_totalMessage.append("\n  ").append(GT.tr("Position: {0}", l_message));
-            l_message = (String)m_mesgParts.get(WHERE);
-            if (l_message != null)
-                l_totalMessage.append("\n  ").append(GT.tr("Where: {0}", l_message));
-        }
+
+        l_message = (String)m_mesgParts.get(HINT);
+        if (l_message != null)
+            l_totalMessage.append("\n  ").append(GT.tr("Hint: {0}", l_message));
+        l_message = (String)m_mesgParts.get(POSITION);
+        if (l_message != null)
+            l_totalMessage.append("\n  ").append(GT.tr("Position: {0}", l_message));
+        l_message = (String)m_mesgParts.get(WHERE);
+        if (l_message != null)
+            l_totalMessage.append("\n  ").append(GT.tr("Where: {0}", l_message));
+
         if (verbosity > 2)
         {
             String l_internalQuery = (String)m_mesgParts.get(INTERNAL_QUERY);
